package org.nakedobjects.nof.reflect.java.annotations;

import java.lang.reflect.Method;

import junit.framework.TestCase;

import org.nakedobjects.noa.annotations.When;
import org.nakedobjects.nof.reflect.java.reflect.JavaObjectForReflector;

public class JavaIntrospectorExtensionTest extends TestCase {
    private static final Class ANNOTATED_CLASS = JavaObjectForReflectorWithExtensions.class;
    private static final Class NON_ANNOTATED_CLASS = JavaObjectForReflector.class;
    private JavaIntrospectorAnnotationExtension annotated;
    private JavaIntrospectorAnnotationExtension nonAnnotated;
    private JavaObjectForReflectorWithExtensions target;
       
    protected void setUp() throws Exception {
         annotated = new JavaIntrospectorAnnotationExtension(ANNOTATED_CLASS);
         nonAnnotated = new JavaIntrospectorAnnotationExtension(NON_ANNOTATED_CLASS);
         target = new JavaObjectForReflectorWithExtensions();
    }

    public void testBounded() throws Exception {
        assertFalse(nonAnnotated.isBounded());
        assertTrue(annotated.isBounded());
    }

    public void testDebugType() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("stop", new Class[0]);
        assertFalse(nonAnnotated.isDebug(method));
        
        method = ANNOTATED_CLASS.getMethod("debugMethod", new Class[0]);
        assertTrue(annotated.isDebug(method));
    }

    public void testExplorationType() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("stop", new Class[0]);
        assertFalse(nonAnnotated.isExploration(method));
        
        method = ANNOTATED_CLASS.getMethod("explorationMethod", new Class[0]);
        assertTrue(annotated.isExploration(method));
    }

    public void testFieldNotPersisted() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("getOne", new Class[0]);
        assertTrue(nonAnnotated.isFieldPersisted(method));
        
        method = ANNOTATED_CLASS.getMethod("getOne", new Class[0]);
        assertFalse(annotated.isFieldPersisted(method));
    }
    
    public void testHiddenMethod() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("stop", new Class[0]);
        assertEquals(When.NEVER, nonAnnotated.isHidden(method));
        
        method = ANNOTATED_CLASS.getMethod("hiddenMethod", new Class[0]);
        assertEquals(When.ALWAYS, annotated.isHidden(method));
    }
    
    public void testImmutableOncePersisted() throws Exception {
        assertFalse(nonAnnotated.isImmutableOncePersisted());
        assertTrue(annotated.isImmutableOncePersisted());
    }
    
    public void testAlwaysImmutable() throws Exception {
        assertFalse(nonAnnotated.isAlwaysImmutable());
        JavaIntrospectorAnnotationExtension annotated2 = new JavaIntrospectorAnnotationExtension(JavaObjectForReflectorWithExtensions2.class);
        assertTrue(annotated2 .isAlwaysImmutable());
    }

    public void testLocal() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("stop", new Class[0]);
        assertFalse(nonAnnotated.isLocal(method));
        
        method = ANNOTATED_CLASS.getMethod("localMethod", new Class[0]);
        assertTrue(annotated.isLocal(method));
    }

    public void testRemote() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("stop", new Class[0]);
        assertFalse(nonAnnotated.isRemote(method));
        
        method = ANNOTATED_CLASS.getMethod("remoteMethod", new Class[0]);
        assertTrue(annotated.isRemote(method));
    }
    
    public void testActionUnusable() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("stop", new Class[0]);
        assertEquals(When.NEVER, nonAnnotated.isActionDisabled(method));
        
        method = ANNOTATED_CLASS.getMethod("unusableMethod", new Class[0]);
        assertEquals(When.ALWAYS, annotated.isActionDisabled(method));
    }

    public void testClassActionOrder() throws Exception {
        assertNull(nonAnnotated.getClassActionOrder());
        
        //assertEquals("1, 2, 3", annotated.getClassActionOrder());
    }
    
    public void testCollectionType() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("getFive", new Class[0]);
        assertNull(nonAnnotated.getCollectionType(method));
        
        method = ANNOTATED_CLASS.getMethod("getCollection", new Class[0]);
        assertEquals(Long.class, annotated.getCollectionType(method));
    }

    public void testObjectActionOrder() throws Exception {
        assertNull(nonAnnotated.getObjectActionOrder());
        
        assertEquals("1, 2, 3", annotated.getObjectActionOrder());
    }

    public void testFieldOrder() throws Exception {
        assertNull(nonAnnotated.getFieldOrder());
        
        assertEquals("4, 5, 6", annotated.getFieldOrder());
    }


    public void testMemberDescription() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("getFive", new Class[0]);
        assertNull(nonAnnotated.getMemberDescription(method));
        
        method = ANNOTATED_CLASS.getMethod("getCollection", new Class[0]);
        assertEquals("description text", annotated.getMemberDescription(method).execute(target));
    }

    public void testMemberName() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("getFive", new Class[0]);
        assertNull(nonAnnotated.getMemberName(method));
        
        method = ANNOTATED_CLASS.getMethod("getCollection", new Class[0]);
        assertEquals("name text", annotated.getMemberName(method).execute(target));
    }

    public void testParameterDefault() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("start", new Class[] { String.class });
        assertNull(nonAnnotated.getParameterDefaults(method));
        
        method = ANNOTATED_CLASS.getMethod("actionMethod", new Class[] { String.class });
        assertEquals(null, annotated.getParameterDefaults(method));
    }

    public void testParameterNames() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("start", new Class[] { String.class });
        assertNull(nonAnnotated.getParameterNames(method));
        
        method = ANNOTATED_CLASS.getMethod("actionMethod", new Class[] { String.class });
        String[] names = (String[]) annotated.getParameterNames(method).execute(target);
        assertEquals("one", names[0]);
        assertEquals(1, names.length);
    }
    
    public void testParameterOptions() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("start", new Class[] { String.class });
        assertNull(nonAnnotated.getParameterOptions(method));
        
        method = ANNOTATED_CLASS.getMethod("actionMethod", new Class[] { String.class });
        assertEquals(null, annotated.getParameterOptions(method));
    }

    public void testPluralName() throws Exception {
        assertNull(nonAnnotated.getPluralName());
        
        assertEquals("plural name", annotated.getPluralName());
    }

    public void testRequiredParameters() throws Exception {
        Method method = NON_ANNOTATED_CLASS.getMethod("start", new Class[] { String.class });
        assertNull(nonAnnotated.getParameterNames(method));
        
        method = ANNOTATED_CLASS.getMethod("actionMethod", new Class[] { String.class });
        boolean[] names = (boolean[]) annotated.getOptionalParameters(method).execute(target);
        assertEquals(true, names[0]);
        assertEquals(1, names.length);
    }

    public void testSingularName() throws Exception {
        assertNull(nonAnnotated.getSingularName());
        
        assertEquals("singular name", annotated.getSingularName());
    }
    
    public void testMultiLineForActionParamIfNot() throws Exception {

        Method method = ANNOTATED_CLASS.getMethod("actionWithStringParams", new Class[] { String.class, String.class });
        int[] parametersNoLines = annotated.getParametersNoLines(method);

        assertEquals(1, parametersNoLines[0]);
        assertEquals(10, parametersNoLines[1]);
    }

}


// Copyright (c) Naked Objects Group Ltd.
