package org.nakedobjects.nof.reflect.java.reflect;

import java.util.Enumeration;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.Persistable;
import org.nakedobjects.noa.adapter.ResolveState;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.nof.core.reflect.Allow;
import org.nakedobjects.testing.DummyNakedObject;
import org.nakedobjects.testing.TestSession;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSystem;


public class JavaIntrospectorTest extends TestCase {

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(new TestSuite(JavaIntrospectorTest.class));
    }

    private JavaIntrospector introspector;
    private TestSystem system;

    private TestSpecification intSpecification;
    private NakedObject adapter;
    private JavaObjectForReflector target;

    private JavaAction findAction(String name) {
        OrderSet objectActions = introspector.getObjectActions();
        Enumeration elements = objectActions.elements();
        while (elements.hasMoreElements()) {
            JavaAction action = (JavaAction) elements.nextElement();
            if (action.getIdentifier().getName().equals(name)) {
                return action;
            }
        }
        fail("action not found: " + name);
        return null;
    }
    
    private JavaField findField(String name) {
        OrderSet fields = introspector.getFields();
        Enumeration elements = fields.elements();
        while (elements.hasMoreElements()) {
            JavaField field = (JavaField) elements.nextElement();
            if (field.getIdentifier().getName().equals(name)) {
                return field;
            }
        }
        fail("action not found: " + name);
        return null;
    }

    protected void setUp() throws ClassNotFoundException {
        LogManager.getLoggerRepository().setThreshold(Level.OFF);

        system = new TestSystem();
        system.init();
        system.addSpecificationToLoader(new TestSpecification());
        system.addSpecificationToLoader(new TestSpecification(String.class.getName()));
        intSpecification = new TestSpecification(int.class.getName());
        system.addSpecificationToLoader(intSpecification);

        introspector = new JavaIntrospector(JavaObjectForReflector.class, null, new DummyBuilder(), new JavaReflector(), null);
        introspector.introspect();
        
        target = new JavaObjectForReflector();
        adapter = new DummyNakedObject(target, ResolveState.RESOLVED);
        ((DummyNakedObject) adapter).setupSpecification(new TestSpecification());
    }

    protected void tearDown() throws Exception {
        system.shutdown();
    }
    
    public void testActionParameterCount() {
        JavaAction action = findAction("start");
        assertEquals(1, action.getParameterCount());
        
        action = findAction("stop");
        assertEquals(0, action.getParameterCount());
    }
    
    public void testActionParameterDefaults() {
        DummyNakedObject adaptedObject = new DummyNakedObject(new JavaObjectForReflector());

        JavaAction action = findAction("start");
        Object[] defaults = action.getParameterDefaults(adaptedObject);
        assertEquals(1, defaults.length);
        assertEquals("default param", defaults[0]);
        
        action = findAction("stop");
        assertEquals(null, action.getParameterDefaults(adaptedObject));
    }

    public void testActionParameterNames() {
        JavaAction action = findAction("start");
        assertEquals(1, action.getParameterNames().length);
        assertEquals("parameter name", action.getParameterNames()[0]);

        action = findAction("stop");
        assertEquals(null, action.getParameterNames());
    }


    public void testActionParameterOptionsViaTypedArray() {
        DummyNakedObject adaptedObject = new DummyNakedObject(new JavaObjectForReflector());

        JavaAction action = findAction("start");
        Object[][] parameterOptions = action.getParameterOptions(adaptedObject);
        assertEquals(1, parameterOptions.length);
        assertEquals(3, parameterOptions[0].length);
        assertEquals("one", parameterOptions[0][0]);
        assertEquals("two", parameterOptions[0][1]);
        assertEquals("three", parameterOptions[0][2]);
    }
    

    public void testActionParameterOptionsViaObjectArray() {
        DummyNakedObject adaptedObject = new DummyNakedObject(new JavaObjectForReflector());

        JavaAction action = findAction("start2");
        Object[][] parameterOptions = action.getParameterOptions(adaptedObject);
        assertEquals(1, parameterOptions.length);
        assertEquals(3, parameterOptions[0].length);
        assertEquals("three", parameterOptions[0][0]);
        assertEquals("two", parameterOptions[0][1]);
        assertEquals("one", parameterOptions[0][2]);
    }

    
    public void testActionParameterNoOptions() {
        DummyNakedObject adaptedObject = new DummyNakedObject(new JavaObjectForReflector());

        JavaAction action = findAction("stop");
        Object[][] parameterOptions = action.getParameterOptions(adaptedObject);
        assertTrue(parameterOptions instanceof Object[][]);
        assertEquals(0, parameterOptions.length);
    }

    public void testActionParameterRequired() {
        JavaAction action = findAction("start");
        assertEquals(false, action.getOptionalParameters()[0]);
        assertEquals(1, action.getOptionalParameters().length);

        action = findAction("two");;
        assertEquals(null, action.getOptionalParameters());
    }

    public void testActionReturnTypes() throws Exception {
        JavaAction action = findAction("stop");
        assertEquals(null, action.getReturnType());

        action = findAction("start");
        assertEquals(intSpecification, action.getReturnType());
    }

    public void testActionSortOrder() {
        OrderSet actions = introspector.getObjectActions();
        Enumeration elements = actions.elements();
        assertEquals("start", ((JavaAction) elements.nextElement()).getIdentifier().getName());
        assertEquals("stop", ((JavaAction) elements.nextElement()).getIdentifier().getName());
        assertEquals("start2", ((JavaAction) elements.nextElement()).getIdentifier().getName());
        assertEquals("hiddenToUser", ((JavaAction) elements.nextElement()).getIdentifier().getName());
    }

    public void testActionWithDefaultTarget() throws Exception {
        JavaAction action = findAction("two");;
        assertEquals(NakedObjectAction.DEFAULT, action.getTarget());
    }

    public void testActionWithRemoteTarget() {
        JavaAction action = findAction("runOnServer");
        assertEquals(NakedObjectAction.REMOTE, action.getTarget());
    }

    public void testActionWithTypeOfDebug() throws Exception {
        JavaAction action = findAction("two");;
        assertEquals(NakedObjectAction.DEBUG, action.getType());
    }

    public void testActionWithTypeOfExploration() throws Exception {
        JavaAction action = findAction("setUp");
        assertEquals(NakedObjectAction.EXPLORATION, action.getType());
    }

    public void testActionWithTypeOfUser() throws Exception {
        JavaAction action = findAction("stop");
        assertEquals(NakedObjectAction.USER, action.getType());
    }

    public void testClassActionDescription() {
        OrderSet actions = introspector.getClassActions();
        Enumeration elements = actions.elements();

        JavaAction javaAction = (JavaAction) elements.nextElement();
        assertEquals("class action description", javaAction.getDescription());
        javaAction = (JavaAction) elements.nextElement();
        assertEquals("", (javaAction).getDescription());
    }

    public void testClassActionList() {
        OrderSet actions = introspector.getClassActions();
        assertEquals(2, actions.size());
    }

    public void testClassActionName() {
        OrderSet actions = introspector.getClassActions();
        Enumeration elements = actions.elements();

        assertEquals("class action name", ((JavaAction) elements.nextElement()).getName());
        assertEquals(null, ((JavaAction) elements.nextElement()).getName());
    }

    public void testClassActionSortOrder() {
        OrderSet actions = introspector.getClassActions();
        Enumeration elements = actions.elements();

        assertEquals("top", ((JavaAction) elements.nextElement()).getIdentifier().getName());
        assertEquals("bottom", ((JavaAction) elements.nextElement()).getIdentifier().getName());
    }

    public void testClassActionValidate() {
        OrderSet actions = introspector.getClassActions();
        Enumeration elements = actions.elements();

        JavaAction action = ((JavaAction) elements.nextElement());
        assertEquals(Allow.DEFAULT, action.isParameterSetValidImperatively(null, new Naked[0]));
        JavaObjectForReflector.classActionValid = "not now";
        assertEquals(true, action.isParameterSetValidImperatively(null, new Naked[0]).isVetoed());
        assertEquals("not now", action.isParameterSetValidImperatively(null, new Naked[0]).getReason());

        action = ((JavaAction) elements.nextElement());
        assertEquals(Allow.DEFAULT, action.isParameterSetValidImperatively(null, new Naked[0]));
        assertEquals(Allow.DEFAULT, action.isParameterSetValidImperatively(null, new Naked[0]));
    }

    public void testFields() throws Exception {
        OrderSet fields = introspector.getFields();
        assertEquals(7, fields.size());
    }

    public void testPersistentField() throws Exception {
        JavaField field = findField("three");
        assertFalse(field.isPersisted());
        
        field = findField("one");
        assertTrue(field.isPersisted());
    }

    public void testHiddenField() throws Exception {
        JavaField field = findField("one");
        TestSession testSession = new TestSession();
        assertEquals(false, field.isUsableForSession(testSession).isAllowed());
        
        field = findField("six");
        assertEquals(true, field.isUsableForSession(testSession).isAllowed());
    }

    public void testProtectedField() throws Exception {
        JavaField field = findField("one");
        assertEquals(Allow.DEFAULT, field.isUsable(adapter));
        
        field = findField("eight");
        Consent usable = field.isUsableDeclaratively();
        assertTrue(usable.isVetoed());
        assertEquals("Field not editable", usable.getReason());
    }
    
    public void testHideField() throws Exception {
        JavaField field = findField("one");
        assertEquals(true, field.isVisible(adapter));
        
        field = findField("fieldTwo");
        assertEquals(false, field.isVisible(adapter));
    }
    
    public void testMandatoryField() throws Exception {
        JavaField field = findField("six");
        assertEquals(true, field.isMandatory());
        
        field = findField("one");
        assertEquals(false, field.isMandatory());
    }
    
    public void testFieldDefault() throws Exception {
        JavaValueAssociation field = (JavaValueAssociation) findField("six");
        assertEquals(null, field.getDefault(adapter));
        
        field = (JavaValueAssociation) findField("one");
        assertEquals("default value", field.getDefault(adapter).getObject());
    }
    
    
    public void testFieldOptions() throws Exception {
        JavaField field = findField("six");
        Object[] options = field.getOptions(adapter);
        assertTrue(options instanceof Object[]);
        assertEquals(2, options.length);
    }
    
    public void testNoFieldOptions() throws Exception {
        JavaField field = findField("five");
        Object[] options = field.getOptions(adapter);
        assertNull(options);
    }


    public void testUnusableField() throws Exception {
        JavaField field = findField("one");
        assertEquals(Allow.DEFAULT, field.isUsable(adapter));
        
        field = findField("seven");
        Consent usable = field.isUsable(adapter);
        assertEquals(true, usable.isVetoed());
        assertEquals("no changes", usable.getReason());
    }

    public void testFieldSortOrder() {
        OrderSet fields = introspector.getFields();
        Enumeration elements = fields.elements();
        assertEquals("one", ((JavaField) elements.nextElement()).getIdentifier().getName());
        assertEquals("fieldTwo", ((JavaField) elements.nextElement()).getIdentifier().getName());
        assertEquals("three", ((JavaField) elements.nextElement()).getIdentifier().getName());
        assertEquals("five", ((JavaField) elements.nextElement()).getIdentifier().getName());
    }

    public void testFieldsUserVisble() {
        OrderSet fields = introspector.getFields();
        Enumeration elements = fields.elements();

        TestSession session = new TestSession();
        assertEquals(true, ((JavaField) elements.nextElement()).isVisibleForSession(session));
        assertEquals(false, ((JavaField) elements.nextElement()).isVisibleForSession(session));
        assertEquals(true, ((JavaField) elements.nextElement()).isVisibleForSession(session));
    }

    public void testFieldsUserUnusable() {
        OrderSet fields = introspector.getFields();
        Enumeration elements = fields.elements();

        TestSession session = new TestSession();
        Consent usable = ((JavaField) elements.nextElement()).isUsableForSession(session);
        assertTrue(usable.isVetoed());
        assertEquals("no edits", usable.getReason());
        assertEquals(Allow.DEFAULT, ((JavaField) elements.nextElement()).isUsableForSession(session));
        
        usable = ((JavaField) elements.nextElement()).isUsableDeclaratively();
        assertTrue(usable.isVetoed());
        assertEquals("Field not editable", usable.getReason());
    }

    public void testInterfaces() {
        String[] interfaces = introspector.getInterfaces();
        assertEquals(2, interfaces.length);
        assertEquals(Interface1.class.getName(), interfaces[0]);
        assertEquals(Interface2.class.getName(), interfaces[1]);
    }

    public void testLocalMethod() {
        JavaAction action = findAction("runOnClient");
        assertEquals(NakedObjectAction.LOCAL, action.getTarget());
    }

    public void testLookup() {
        assertFalse(introspector.isBounded());
    }

    public void testNameManipulations() {
        assertEquals("CarRegistration", JavaIntrospector.javaBaseName("getCarRegistration"));
        assertEquals("Driver", JavaIntrospector.javaBaseName("Driver"));
        assertEquals("Register", JavaIntrospector.javaBaseName("actionRegister"));
        assertEquals("", JavaIntrospector.javaBaseName("action"));
    }

    public void testObjectActionDescription() {
        OrderSet actions = introspector.getObjectActions();
        Enumeration elements = actions.elements();
        JavaAction javaAction = (JavaAction) elements.nextElement();
        assertEquals("", (javaAction).getDescription());
        javaAction = (JavaAction) elements.nextElement();
        assertEquals("object action description", javaAction.getDescription());
    }

    public void testObjectActionList() {
        OrderSet actions = introspector.getObjectActions();
        assertEquals(8, actions.size());
    }

    public void testObjectActionMethodsUserVisble() {
        OrderSet actions = introspector.getObjectActions();
        Enumeration elements = actions.elements();

        TestSession session = new TestSession();
        assertEquals(true, ((JavaAction) elements.nextElement()).isVisibleForSession(session));
        assertEquals(true, ((JavaAction) elements.nextElement()).isVisibleForSession(session));
        assertEquals(true, ((JavaAction) elements.nextElement()).isVisibleForSession(session));
        assertEquals(false, ((JavaAction) elements.nextElement()).isVisibleForSession(session));
    }

    public void testObjectActionMethodsRequiredParameters() {
        OrderSet actions = introspector.getObjectActions();
        Enumeration elements = actions.elements();

        JavaAction action = ((JavaAction) elements.nextElement());
        assertEquals(1, action.getOptionalParameters().length);
        assertEquals(false, action.getOptionalParameters()[0]);

        action = ((JavaAction) elements.nextElement());
        assertEquals(null, action.getOptionalParameters());

        action = ((JavaAction) elements.nextElement());
        assertEquals(null, action.getOptionalParameters());
    }

    public void testObjectActionName() {
        OrderSet actions = introspector.getObjectActions();
        Enumeration elements = actions.elements();
        assertEquals(null, ((JavaAction) elements.nextElement()).getName());
        assertEquals("object action name", ((JavaAction) elements.nextElement()).getName());
    }

    public void testObjectActionValidate() {
        OrderSet actions = introspector.getObjectActions();
        Enumeration elements = actions.elements();

        JavaAction action = ((JavaAction) elements.nextElement());
        assertEquals(Allow.DEFAULT, action.isParameterSetValidImperatively(adapter, new Naked[1]));
        target.objectActionValid = "not valid";
        assertEquals(true, action.isParameterSetValidImperatively(adapter, new Naked[1]).isVetoed());
        assertEquals("not valid", action.isParameterSetValidImperatively(adapter, new Naked[1]).getReason());

        action = ((JavaAction) elements.nextElement());
        assertEquals(Allow.DEFAULT, action.isParameterSetValidImperatively(adapter, new Naked[1]));
        assertEquals(Allow.DEFAULT, action.isParameterSetValidImperatively(adapter, new Naked[1]));

    }

    public void testPersistable() {
        assertEquals(Persistable.USER_PERSISTABLE, introspector.persistable());

        introspector = new JavaIntrospector(JavaObjectForReflectorTransient.class, null, new DummyBuilder(), new JavaReflector(), null);
        introspector.introspect();
        assertEquals(Persistable.TRANSIENT, introspector.persistable());

    }

    public void testPluralName() {
        assertEquals("Plural", introspector.pluralName());
    }

    public void testShortName() {
        assertEquals("JavaObjectForReflector", introspector.shortName());
    }

    public void testSingularName() {
        assertEquals("Singular", introspector.singularName());
    }

    public void testSuperclass() {
        assertEquals(Object.class.getName(), introspector.getSuperclass());
    }
}
// Copyright (c) Naked Objects Group Ltd.
