package org.nakedobjects.nof.reflect.java.value;

import java.util.Locale;

import org.nakedobjects.applib.value.Money;
import org.nakedobjects.noa.adapter.TextEntryParseException;
import org.nakedobjects.nof.core.adapter.value.AbstractValueAdapterTestCase;


public class MoneyAdapterTest extends AbstractValueAdapterTestCase {

    private static final String POUND_SYMBOL = "\u00A3";
    private static final String EURO_SYMBOL = "\u20AC";
    private MoneyAdapter value;

    protected void setUp() throws Exception {
        super.setUp();
        Locale.setDefault(Locale.UK);
        
        super.value = value = new MoneyAdapter(new Money(10.5, "ffr"));
    }

    public void testLocale() {
        assertEquals(Locale.UK, Locale.getDefault());
    }

    public void testClear() {
        value.clear();
        assertTrue(value.isEmpty());
        assertEquals(null, value.getObject());
    }

    public void testEncoding() {
        String data = value.asEncodedString();
        assertEquals("10.5 FFR", new String(data));
    }

    public void testEncodingWhenEmpty() {
        MoneyAdapter adapter = new MoneyAdapter();
        String data = adapter.asEncodedString();
        assertEquals("NULL", new String(data));
    }

    public void testDecodingWhenEmpty() {
        value.restoreFromEncodedString("NULL");
        assertTrue(value.isEmpty());
    }

    public void testDecoding() {
        value.restoreFromEncodedString("23.77 FFR");
        Money expected = new Money(23.77, "FFR");
        assertEquals(expected, value.getObject());
    }

    public void testTitleWithPounds() {
        MoneyAdapter value = new MoneyAdapter(new Money(10.5, "gbp"));
        assertEquals(POUND_SYMBOL + "10.50", value.titleString());
    }

    public void testTitleWithNonLocalCurrency() {
        MoneyAdapter adapter = new MoneyAdapter(new Money(10.50, "usd"));
        assertEquals("10.50 USD", adapter.titleString());
    }

    public void testTitleWithUnknownCurrency() {
        MoneyAdapter adapter = new MoneyAdapter(new Money(10.50, "UNK"));
        assertEquals("10.50 UNK", adapter.titleString());
    }

    public void testTitleWithNoValueShowsBlank() {
        MoneyAdapter adapter = new MoneyAdapter();
        assertEquals("", adapter.titleString());
    }

    public void testUserEntry() {
        value.parseTextEntry("22.45 USD");
        assertEquals(new Money(22.45, "usd"), value.getObject());
    }

    public void testUserEntryUsesPreviousCurrency() {
        MoneyAdapter adapter = new MoneyAdapter(new Money(10.50, "cad"));
        adapter.parseTextEntry("22.45");
        assertEquals(new Money(22.45, "cad"), adapter.getObject());
    }

    public void testReplacementEntryForDefaultCurrency() {
        MoneyAdapter adapter = new MoneyAdapter(new Money(10.3, "gbp"));
        adapter.parseTextEntry(POUND_SYMBOL + "80.90");
        assertEquals(new Money(80.90, "gbp"), adapter.getObject());
    }

    public void testSpecifyingCurrencyInEntry() {
        MoneyAdapter adapter = new MoneyAdapter();
        adapter.parseTextEntry("3021.50 ffr");
        assertEquals("3,021.50 FFR", adapter.titleString());
    }

    public void testUsingLocalCurrencySymbol() {
        MoneyAdapter adapter = new MoneyAdapter(new Money(0L, "gbp"));
        adapter.parseTextEntry(POUND_SYMBOL + "3021.50");
        assertEquals(POUND_SYMBOL + "3,021.50", adapter.titleString());
    }

    public void testInvalidCurrencyCodeIsRejected() throws Exception {
        MoneyAdapter adapter = new MoneyAdapter(new Money(1, "EUR"));
        try {
            adapter.parseTextEntry("3021.50  Dm");
            fail("invalid code accepted " + adapter);
        } catch (TextEntryParseException expected) {}
    }

    public void testInvalidCurrencySymbolIsRejected() throws Exception {
        MoneyAdapter adapter = new MoneyAdapter(new Money(1, "EUR"));
        try {
            adapter.parseTextEntry(POUND_SYMBOL + "3021.50");
            fail("invalid code accepted " + adapter);
        } catch (TextEntryParseException expected) {}
    }

    public void testValidCurrencySymbolIsIgnore() throws Exception {
        MoneyAdapter adapter = new MoneyAdapter(new Money(1, "EUR"));
        adapter.parseTextEntry("3021.50" + EURO_SYMBOL);
        assertEquals("3,021.50 EUR", adapter.titleString());
    }

    public void testNewValueDefaultsToLocalCurrency() throws Exception {
        MoneyAdapter adapter = new MoneyAdapter();
        adapter.parseTextEntry("3021.50");
        assertEquals(POUND_SYMBOL + "3,021.50", adapter.titleString());
    }

    public void testUnrelatedCurrencySymbolIsRejected() throws Exception {
        MoneyAdapter adapter = new MoneyAdapter(new Money(1, "eur"));
        try {
            adapter.parseTextEntry("$3021.50");
            fail("invalid code accepted " + adapter);
        } catch (TextEntryParseException expected) {}
    }

}

// Copyright (c) Naked Objects Group Ltd.
